@@ -9,8 +9,6 @@
 #include "wv.h"
 #include "getopt.h"
 
-#define VERSION "0.6.5"
-
 /* strdup isn't declared in <string.h> for `gcc -ansi'; declare it here */
 extern char *strdup (const char *);
 
@@ -40,11 +38,16 @@ int mydochandler (wvParseStruct * ps, wvTag tag);
 int myCharProc (wvParseStruct * ps, U16 eachchar, U8 chartype, U16 lid);
 int mySpecCharProc (wvParseStruct * ps, U16 eachchar, CHP * achp);
 
-FILE *wvOpenConfig (char *config);
+int wvOpenConfig (state_data *myhandle,char *config);
 
+char * wv_arg_basename = 0;
+char * figure_name (wvParseStruct * ps);
+char * name_to_url (char * name);
 
-int HandleBitmap (char *name, BitmapBlip * bitmap);
-int HandleMetafile (char *name, MetaFileBlip * bitmap);
+char wv_cwd[4097];
+
+int HandleBitmap (wvParseStruct * ps, char *name, BitmapBlip * bitmap);
+int HandleMetafile (wvParseStruct * ps, char *name, MetaFileBlip * bitmap);
 
 /* should really be a config.h decl for having strdup, but... */
 #ifdef __MWERKS__
@@ -67,27 +70,13 @@ strdup (const char *text)
 char *
 wvHtmlGraphic (wvParseStruct * ps, Blip * blip)
 {
-    static int i;
-    char buffer[10];
     char *name;
-    FILE *fd;
+    wvStream * fd;
     char test[3];
-    sprintf (buffer, "%d", i++);
-    name = strdup (ps->filename);
-    wvTrace (("name is %s\n", name));
-    remove_suffix (name, ".doc");
-    if (ps->dir != NULL)
-      {
-	  char *tempa, *tempb;
-	  tempb = strdup (ps->dir);
-	  tempa = base_name (name);
-	  wvAppendStr (&tempb, "/");
-	  wvAppendStr (&tempb, tempa);
-	  wvFree (name);
-	  name = tempb;
-      }
 
-    wvAppendStr (&name, buffer);
+    name = figure_name (ps);
+    if (name == 0) return (0);
+
     /* 
        temp hack to test older included bmps in word 6 and 7,
        should be wrapped in a modern escher strucure before getting
@@ -99,24 +88,21 @@ wvHtmlGraphic (wvParseStruct * ps, Blip * blip)
       case msoblipJPEG:
       case msoblipDIB:
       case msoblipPNG:
-	  fd = (FILE *) (blip->blip.bitmap.m_pvBits);
+	  fd =  (blip->blip.bitmap.m_pvBits);
 	  test[2] = '\0';
-	  test[0] = getc (fd);
-#if 1
-	  test[1] = getc (fd);
-	  rewind (fd);
+	  test[0] = read_8ubit (fd);
+
+	  test[1] = read_8ubit (fd);
+	  wvStream_rewind (fd);
 	  if (!(strcmp (test, "BM")))
-#else
-	  test[1] = '\0';
-	  ungetc (test[1], fd);
-	  if (!(strcmp (test, "B")))
-#endif
 	    {
 		wvAppendStr (&name, ".bmp");
-		if (0 != HandleBitmap (name, &blip->blip.bitmap))
+		if (0 != HandleBitmap (ps, name, &blip->blip.bitmap))
 		    return (NULL);
 		remove_suffix (name, ".bmp");
+		if (ps->dir) chdir (ps->dir);
 		bmptopng (name);
+		if (ps->dir) chdir (wv_cwd);
 		wvAppendStr (&name, ".png");
 		return (name);
 	    }
@@ -128,32 +114,32 @@ wvHtmlGraphic (wvParseStruct * ps, Blip * blip)
       {
       case msoblipWMF:
 	  wvAppendStr (&name, ".wmf");
-	  if (0 != HandleMetafile (name, &blip->blip.metafile))
+	  if (0 != HandleMetafile (ps, name, &blip->blip.metafile))
 	      return (NULL);
 	  break;
       case msoblipEMF:
 	  wvAppendStr (&name, ".emf");
-	  if (0 != HandleMetafile (name, &blip->blip.metafile))
+	  if (0 != HandleMetafile (ps, name, &blip->blip.metafile))
 	      return (NULL);
 	  break;
       case msoblipPICT:
 	  wvAppendStr (&name, ".pict");
-	  if (0 != HandleMetafile (name, &blip->blip.metafile))
+	  if (0 != HandleMetafile (ps, name, &blip->blip.metafile))
 	      return (NULL);
 	  break;
       case msoblipJPEG:
 	  wvAppendStr (&name, ".jpg");
-	  if (0 != HandleBitmap (name, &blip->blip.bitmap))
+	  if (0 != HandleBitmap (ps, name, &blip->blip.bitmap))
 	      return (NULL);
 	  break;
       case msoblipDIB:
 	  wvAppendStr (&name, ".dib");
-	  if (0 != HandleBitmap (name, &blip->blip.bitmap))
+	  if (0 != HandleBitmap (ps, name, &blip->blip.bitmap))
 	      return (NULL);
 	  break;
       case msoblipPNG:
 	  wvAppendStr (&name, ".png");
-	  if (0 != HandleBitmap (name, &blip->blip.bitmap))
+	  if (0 != HandleBitmap (ps, name, &blip->blip.bitmap))
 	      return (NULL);
 	  break;
       }
@@ -162,18 +148,25 @@ wvHtmlGraphic (wvParseStruct * ps, Blip * blip)
 
 
 int
-HandleBitmap (char *name, BitmapBlip * bitmap)
+HandleBitmap (wvParseStruct * ps, char *name, BitmapBlip * bitmap)
 {
-    int c;
-    FILE *fd;
+    wvStream * pwv = bitmap->m_pvBits;
+    FILE *fd = NULL;
+    size_t size = 0, i;
+
+    if (ps->dir) chdir (ps->dir);
     fd = fopen (name, "wb");
+    if (ps->dir) chdir (wv_cwd);
     if (fd == NULL)
       {
-	  wvError (("Cannot open %s for writing:%s\n", name, strerror (errno)));
-	  return (-1);
+	fprintf (stderr,"\nCannot open %s for writing:%s\n",name,strerror (errno));
+	exit (1);
       }
-    while (EOF != (c = getc ((FILE *) (bitmap->m_pvBits))))
-	fputc (c, fd);
+    size = wvStream_size (pwv);
+    wvStream_rewind(pwv);
+
+    for (i = 0; i < size; i++)
+      fputc (read_8ubit(pwv), fd);
     fclose (fd);
     wvTrace (("Name is %s\n", name));
     return (0);
@@ -181,18 +174,25 @@ HandleBitmap (char *name, BitmapBlip * bitmap)
 
 
 int
-HandleMetafile (char *name, MetaFileBlip * bitmap)
+HandleMetafile (wvParseStruct * ps, char *name, MetaFileBlip * bitmap)
 {
-    int c;
-    FILE *fd;
+    wvStream * pwv = bitmap->m_pvBits;
+    FILE *fd = NULL;
+    size_t size = 0, i;
+
+    if (ps->dir) chdir (ps->dir);
     fd = fopen (name, "wb");
+    if (ps->dir) chdir (wv_cwd);
     if (fd == NULL)
       {
-	  wvError (("Cannot open %s for writing:%s\n", name, strerror (errno)));
-	  return (-1);
+	fprintf (stderr,"\nCannot open %s for writing:%s\n",name,strerror (errno));
+	exit (1);
       }
-    while (EOF != (c = getc ((FILE *) (bitmap->m_pvBits))))
-	fputc (c, fd);
+    size = wvStream_size (pwv);
+    wvStream_rewind(pwv);
+
+    for (i = 0; i < size; i++)
+      fputc (read_8ubit(pwv), fd);
     fclose (fd);
     wvTrace (("Name is %s\n", name));
     return (0);
@@ -213,9 +213,11 @@ do_help (void)
     printf ("\nCommon Options:\n");
     printf ("  -x --config=config.xml\tSpecify an output filter to use\n");
     printf ("  -c --charset=charset\t\tSpecify an iconv charset encoding\n");
-    printf
-	("  -p --password=password\tSpecify password for encrypted\n\t\t\t\tWord Documents\n");
+    printf ("  -p --password=password\tSpecify password for encrypted\n\t\t\t\tWord Documents\n");
     printf ("  -d --dir=dir\t\t\tDIR\n");
+    printf ("  -b --basename=name\t\tUse name as base name of image files\n");
+    printf ("  -a --auto-eps=fmt\t\tQuery support for conversion of fmt to eps\n");
+    printf ("  -s --suppress=fmt\t\tDon't convert fmt to eps\n");
     printf ("  -v --version\t\t\tPrint wvWare's version number\n");
     printf ("  -? --help\t\t\tPrint this help message\n");
     printf
@@ -230,6 +232,9 @@ do_help (void)
     printf ("Caolan McNamara (original author)\nVisit http://www.wvware.com\n");
 }
 
+static void wv_query_eps (const char* format);
+static void wv_suppress (const char* format);
+
 char *charset = NULL;
 
 int
@@ -248,6 +253,9 @@ main (int argc, char **argv)
 	{"config", 1, 0, 'x'},
 	{"password", 1, 0, 'p'},
 	{"dir", 1, 0, 'd'},
+	{"basename", 1, 0, 'b'},
+	{"auto-eps", 1, 0, 'a'},
+	{"suppress", 1, 0, 's'},
 	{"version", 0, 0, 'v'},
 	{"help", 0, 0, '?'},
 	{0, 0, 0, 0}
@@ -261,7 +269,7 @@ main (int argc, char **argv)
 
     while (1)
       {
-	  c = getopt_long (argc, argv, "?vc:x:p:d:", long_options, &index);
+	  c = getopt_long (argc, argv, "?vc:x:p:d:b:a:s:", long_options, &index);
 	  if (c == -1)
 	      break;
 	  switch (c)
@@ -296,6 +304,18 @@ main (int argc, char **argv)
 		else
 		    wvError (("No directory given to dir option"));
 		break;
+	    case 'b':
+		if (optarg)
+		    wv_arg_basename = optarg;
+		else
+		    wvError (("No name given to basename option"));
+		break;
+	    case 'a':
+		wv_query_eps (optarg);
+		return 0;
+	    case 's':
+		wv_suppress (optarg);
+		break;
 	    default:
 		do_help ();
 		return -1;
@@ -308,6 +328,7 @@ main (int argc, char **argv)
 	  return (-1);
       }
 
+#if 0
     input = fopen (argv[optind], "rb");
     if (!input)
       {
@@ -316,7 +337,12 @@ main (int argc, char **argv)
 	  return (-1);
       }
     fclose (input);
+#endif
 
+    getcwd (wv_cwd,4096);
+    wv_cwd[4096] = 0;
+
+    wvInit ();
     ret = wvInitParser (&ps, argv[optind]);
     ps.filename = argv[optind];
     ps.dir = dir;
@@ -365,7 +391,7 @@ main (int argc, char **argv)
 
     if (ret)
       {
-	  wvError (("startup error\n"));
+	  wvError (("startup error #%d\n", ret));
 	  wvOLEFree ();
 	  return (-1);
       }
@@ -376,8 +402,8 @@ main (int argc, char **argv)
     wvSetSpecialCharHandler (&ps, mySpecCharProc);
 
     wvInitStateData (&myhandle);
-    myhandle.fp = wvOpenConfig (config);
-    if (myhandle.fp == NULL)
+
+    if (wvOpenConfig (&myhandle,config) == 0)
       {
 	  wvError (("config file not found\n"));
 	  return (-1);
@@ -587,6 +613,8 @@ long wv_wmfTell (void *);
 
 void wvConvert_WMF_to_EPS (int, int, char **);
 void wvConvert_WMF_to_PNG (int, int, char **);
+void wvConvert_PNG_to_EPS (int, int, char **);
+void wvConvert_JPG_to_EPS (int, int, char **);
 
 int
 wv_wmfRead (void *context)
@@ -610,6 +638,9 @@ wv_wmfTell (void *context)
 
 #include <libwmf/api.h>
 #include <libwmf/eps.h>
+#ifdef HAVE_LIBWMF_FOREIGN_H
+#include <libwmf/foreign.h>
+#endif
 
 #endif /* HAVE_LIBWMF */
 
@@ -670,7 +701,9 @@ wvConvert_WMF_to_EPS (int width, int height, char **source)
     if (err != wmf_E_None)
 	goto _wmf_error;
 
-    ddata->out = out;
+    ddata->out = wmf_stream_create (API,out);
+    if (out == 0)
+	goto _wmf_error;
 
     ddata->bbox = bbox;
 
@@ -801,6 +834,338 @@ wvConvert_WMF_to_PNG (int width, int height, char **source)
 #endif /* HAVE_LIBWMF */
 }
 
+void
+wvConvert_PNG_to_EPS (int width, int height, char **source)
+{
+#ifdef HAVE_LIBWMF_FOREIGN_H
+    FILE *in = 0;
+    FILE *out = 0;
+
+    char *sink = 0;
+
+    unsigned long flags;
+
+    wmf_error_t err;
+
+    wmf_foreign_t *ddata = 0;
+
+    wmfAPI *API = 0;
+
+    wmfAPI_Options api_options;
+
+    wmfImage image;
+
+    flags = WMF_OPT_IGNORE_NONFATAL | WMF_OPT_FUNCTION;
+    api_options.function = wmf_foreign_function;
+
+    err = wmf_api_create (&API, flags, &api_options);
+    if (err != wmf_E_None)
+	return;
+
+    ddata = WMF_FOREIGN_GetData (API);
+
+    if ((ddata->flags & WMF_FOREIGN_SUPPORTS_PNG) == 0)
+      {
+      	wmf_api_destroy (API);
+      	return;
+      }
+
+    in = fopen (*source, "rb");
+
+    if (in == 0)
+      {
+      	wmf_api_destroy (API);
+      	return;
+      }
+
+    if (wmf_image_load_png (API,in,&image) == (-1))
+      {
+        fclose (in);
+      	wmf_api_destroy (API);
+      	return;
+      }
+
+    fclose (in);
+
+    sink = strdup (*source);
+
+    remove_suffix (sink, ".png");
+    wvAppendStr (&sink, ".eps");
+
+    out = fopen (sink, "w");
+
+    if (out == 0)
+      {
+        wvFree (sink);
+        wmf_image_free (API,&image);
+        wmf_api_destroy (API);
+        return;
+      }
+
+    wmf_image_save_eps (API,out,&image);
+
+    fclose (out);
+
+    wmf_image_free (API,&image);
+    wmf_api_destroy (API);
+
+    *source = sink;
+
+    return;
+#endif /* HAVE_LIBWMF_FOREIGN_H */
+}
+
+void
+wvConvert_JPG_to_EPS (int width, int height, char **source)
+{
+#ifdef HAVE_LIBWMF_FOREIGN_H
+    FILE *in = 0;
+    FILE *out = 0;
+
+    char *sink = 0;
+
+    unsigned long flags;
+
+    wmf_error_t err;
+
+    wmf_foreign_t *ddata = 0;
+
+    wmfAPI *API = 0;
+
+    wmfAPI_Options api_options;
+
+    wmfImage image;
+
+    flags = WMF_OPT_IGNORE_NONFATAL | WMF_OPT_FUNCTION;
+    api_options.function = wmf_foreign_function;
+
+    err = wmf_api_create (&API, flags, &api_options);
+    if (err != wmf_E_None)
+	return;
+
+    ddata = WMF_FOREIGN_GetData (API);
+
+    if ((ddata->flags & WMF_FOREIGN_SUPPORTS_JPEG) == 0)
+      {
+      	wmf_api_destroy (API);
+      	return;
+      }
+
+    in = fopen (*source, "rb");
+
+    if (in == 0)
+      {
+      	wmf_api_destroy (API);
+      	return;
+      }
+
+    if (wmf_image_load_jpg (API,in,&image) == (-1))
+      {
+        fclose (in);
+      	wmf_api_destroy (API);
+      	return;
+      }
+
+    fclose (in);
+
+    sink = strdup (*source);
+
+    remove_suffix (sink, ".jpg");
+    wvAppendStr (&sink, ".eps");
+
+    out = fopen (sink, "w");
+
+    if (out == 0)
+      {
+        wvFree (sink);
+        wmf_image_free (API,&image);
+        wmf_api_destroy (API);
+        return;
+      }
+
+    wmf_image_save_eps (API,out,&image);
+
+    fclose (out);
+
+    wmf_image_free (API,&image);
+    wmf_api_destroy (API);
+
+    *source = sink;
+
+    return;
+#endif /* HAVE_LIBWMF_FOREIGN_H */
+}
+
+static void wv_query_eps (const char* format)
+{
+#ifdef HAVE_LIBWMF
+  unsigned long flags;
+
+  wmf_error_t err;
+#ifdef HAVE_LIBWMF_FOREIGN_H
+  wmf_foreign_t *ddata = 0;
+#endif /* HAVE_LIBWMF_FOREIGN_H */
+  wmfAPI* API = 0;
+  wmfAPI_Options api_options;
+#endif /* HAVE_LIBWMF */
+
+  if (format == 0)
+    {
+      printf ("no\n");
+      return;
+    }
+
+#ifdef HAVE_LIBWMF
+  if (strcmp (format,"wmf") == 0)
+    {
+      printf ("yes\n");
+      return;
+    }
+#ifdef HAVE_LIBWMF_FOREIGN_H
+  if (strcmp (format,"png") == 0)
+    {
+      flags = WMF_OPT_IGNORE_NONFATAL | WMF_OPT_FUNCTION;
+      api_options.function = wmf_foreign_function;
+
+      err = wmf_api_create (&API, flags, &api_options);
+      if (err != wmf_E_None)
+        {
+          printf ("no\n");
+          return;
+        }
+
+      ddata = WMF_FOREIGN_GetData (API);
+
+      if (ddata->flags & WMF_FOREIGN_SUPPORTS_PNG)
+        {
+          printf ("yes\n");
+        }
+      else
+        {
+          printf ("no\n");
+        }
+
+      wmf_api_destroy (API);
+      return;
+    }
+  if (strcmp (format,"jpg") == 0)
+    {
+      flags = WMF_OPT_IGNORE_NONFATAL | WMF_OPT_FUNCTION;
+      api_options.function = wmf_foreign_function;
+
+      err = wmf_api_create (&API, flags, &api_options);
+      if (err != wmf_E_None)
+        {
+          printf ("no\n");
+          return;
+        }
+
+      ddata = WMF_FOREIGN_GetData (API);
+
+      if (ddata->flags & WMF_FOREIGN_SUPPORTS_JPEG)
+        {
+          printf ("yes\n");
+        }
+      else
+        {
+          printf ("no\n");
+        }
+
+      wmf_api_destroy (API);
+      return;
+    }
+#endif /* HAVE_LIBWMF_FOREIGN_H */
+#endif /* HAVE_LIBWMF */
+
+  printf ("no\n");
+  return;
+}
+
+static int Convert_WMF  = 1;
+static int Convert_EMF  = 1;
+static int Convert_PNG  = 1;
+static int Convert_JPG  = 1;
+static int Convert_PICT = 1;
+
+static void wv_suppress (const char* format)
+{
+  const char* ptr = format;
+
+  if (format == 0)
+    {
+      Convert_WMF  = 1;
+      Convert_EMF  = 1;
+      Convert_PNG  = 1;
+      Convert_JPG  = 1;
+      Convert_PICT = 1;
+
+      return;
+    }
+
+  while (*ptr)
+    {
+      if (strncmp (ptr,"wmf,",4) == 0)
+        {
+          Convert_WMF = 0;
+          ptr += 4;
+          continue;
+        }
+      if (strncmp (ptr,"emf,",4) == 0)
+        {
+          Convert_EMF = 0;
+          ptr += 4;
+          continue;
+        }
+      if (strncmp (ptr,"png,",4) == 0)
+        {
+          Convert_PNG = 0;
+          ptr += 4;
+          continue;
+        }
+      if (strncmp (ptr,"jpg,",4) == 0)
+        {
+          Convert_JPG = 0;
+          ptr += 4;
+          continue;
+        }
+      if (strncmp (ptr,"pict,",5) == 0)
+        {
+          Convert_PICT = 0;
+          ptr += 5;
+          continue;
+        }
+
+      if (strcmp (ptr,"wmf") == 0)
+        {
+          Convert_WMF = 0;
+          break;
+        }
+      if (strcmp (ptr,"emf") == 0)
+        {
+          Convert_EMF = 0;
+          break;
+        }
+      if (strcmp (ptr,"png") == 0)
+        {
+          Convert_PNG = 0;
+          break;
+        }
+      if (strcmp (ptr,"jpg") == 0)
+        {
+          Convert_JPG = 0;
+          break;
+        }
+      if (strcmp (ptr,"pict") == 0)
+        {
+          Convert_PICT = 0;
+          break;
+        }
+
+      fprintf (stderr,"format(s) `%s' not recognized!\n",ptr);
+      break;
+    }
+}
+
 void
 wvPrintGraphics (char *config, int graphicstype, int width, int height,
 		 char *source)
@@ -809,8 +1174,14 @@ wvPrintGraphics (char *config, int graphicstype, int width, int height,
 	|| (strstr (config, "wvCleanLaTeX.xml") != NULL))
       {
 	  if (strlen (source) >= 4)
-	      if (strcmp (source + strlen (source) - 4, ".wmf") == 0)
+	    {
+	      if (Convert_WMF && strcmp (source + strlen (source) - 4, ".wmf") == 0)
 		  wvConvert_WMF_to_EPS (width, height, &source);
+	      else if (Convert_PNG && strcmp (source + strlen (source) - 4, ".png") == 0)
+		  wvConvert_PNG_to_EPS (width, height, &source);
+	      else if (Convert_JPG && strcmp (source + strlen (source) - 4, ".jpg") == 0)
+		  wvConvert_JPG_to_EPS (width, height, &source);
+	    }
 	  remove_suffix (source, ".eps");
 	  remove_suffix (source, ".wmf");
 	  remove_suffix (source, ".pict");
@@ -821,16 +1192,24 @@ wvPrintGraphics (char *config, int graphicstype, int width, int height,
 	   */
 	  printf ("\n\\resizebox{%dpt}{%dpt}\
 		  {\\includegraphics{%s.eps}}\
-		  \n-- %#.2x graphic -- \n", width, height, source, graphicstype);
+		  \n% -- %#.2x graphic -- \n", width, height, source, graphicstype);
       }
     else
       {
 	  if (strlen (source) >= 4)
 	      if (strcmp (source + strlen (source) - 4, ".wmf") == 0)
 		  wvConvert_WMF_to_PNG (width, height, &source);
-	  printf
-	      ("<img width=\"%d\" height=\"%d\" alt=\"%#.2x graphic\" src=\"%s\"><br>",
-	       width, height, graphicstype, source);
+	  if ((strstr (config, "wvHtml.xml") != NULL)
+	   || (strstr (config, "wvWml.xml")  != NULL))
+	    {
+	      printf ("<img width=\"%d\" height=\"%d\" alt=\"%#.2x graphic\" src=\"%s\"><br>",
+	              width, height, graphicstype, name_to_url (source));
+	    }
+	  else
+	    {
+	      printf ("<img width=\"%d\" height=\"%d\" alt=\"%#.2x graphic\" src=\"%s\"><br>",
+	              width, height, graphicstype, source);
+	    }
       }
     return;
 }
@@ -901,7 +1280,7 @@ mySpecCharProc (wvParseStruct * ps, U16 eachchar, CHP * achp)
 
 	      if (achp->fOle2)
 		  exit (139);
-#if 0
+#if 1
 	      wvStream_goto (ps->data, achp->fcPic_fcObj_lTagObj);
 	      wvGetPICF (wvQuerySupported (&ps->fib, NULL), &picf, ps->data);
 	      f = picf.rgb;
@@ -909,10 +1288,12 @@ mySpecCharProc (wvParseStruct * ps, U16 eachchar, CHP * achp)
 		{
 		    wvTrace (("Here\n"));
 		    name = wvHtmlGraphic (ps, &blip);
+		    if (ps->dir) chdir (ps->dir);
 		    wvPrintGraphics (config, 0x01,
 				     (int) wvTwipsToHPixels (picf.dxaGoal),
 				     (int) wvTwipsToVPixels (picf.dyaGoal),
 				     name);
+		    if (ps->dir) chdir (wv_cwd);
 		    wvFree (name);
 		}
 	      else
@@ -946,6 +1327,7 @@ mySpecCharProc (wvParseStruct * ps, U16 eachchar, CHP * achp)
 			    {
 				wvTrace (("Here\n"));
 				name = wvHtmlGraphic (ps, &blip);
+				if (ps->dir) chdir (ps->dir);
 				wvPrintGraphics (config, 0x08,
 						 (int)
 						 wvTwipsToHPixels (fspa->xaRight
@@ -958,6 +1340,7 @@ mySpecCharProc (wvParseStruct * ps, U16 eachchar, CHP * achp)
 									 fspa->
 									 yaTop),
 						 name);
+				if (ps->dir) chdir (wv_cwd);
 				wvFree (name);
 			    }
 			  else
@@ -1007,6 +1390,9 @@ mySpecCharProc (wvParseStruct * ps, U16 eachchar, CHP * achp)
 	      U16 symbol[6] = { 'S', 'y', 'm', 'b', 'o', 'l' };
 	      U16 wingdings[9] =
 		  { 'W', 'i', 'n', 'g', 'd', 'i', 'n', 'g', 's' };
+	      U16 mtextra[8] = 
+		  { 'M', 'T', ' ', 'E', 'x', 't', 'r', 'a' };
+
 	      wvTrace (
 		       ("no of strings %d %d\n", ps->fonts.nostrings,
 			achp->ftcSym));
@@ -1032,6 +1418,30 @@ option to support correct symbol font conversion to a viewable format.\n");
 					 (achp->xchSym - 61440), charset);
 		    return (0);
 		}
+	      else if (0 == 
+		       memcmp (mtextra, ps->fonts.ffn[achp->ftcSym].xszFfn, 
+				16))
+		{
+		    if ((!message) && (strcasecmp ("UTF-8", charset)))
+		      {
+			  wvWarning
+			      ("MT Extra font detected (too late sorry!), rerun wvHtml with option --charset utf-8\n\
+option to support correct symbol font conversion to a viewable format.\n");
+			  message++;
+		      }
+		    wvTrace (
+			     ("Symbol char %d %x %c, using font %d %s\n",
+			      achp->xchSym, achp->xchSym, achp->xchSym,
+			      achp->ftcSym,
+			      wvWideStrToMB (ps->fonts.ffn[achp->ftcSym].
+					     xszFfn)));
+		    wvTrace (
+			     ("symbol char ends up as a unicode %x\n",
+			      wvConvertMTExtraToUnicode (achp->xchSym - 61440)));
+		    wvOutputFromUnicode (wvConvertMTExtraToUnicode
+					 (achp->xchSym - 61440), charset);
+		    return (0);
+		}	
 	      else if (0 ==
 		       memcmp (wingdings, ps->fonts.ffn[achp->ftcSym].xszFfn,
 			       18))
@@ -1121,8 +1531,8 @@ myCharProc (wvParseStruct * ps, U16 eachchar, U8 chartype, U16 lid)
 }
 
 
-FILE *
-wvOpenConfig (char *config)
+int
+wvOpenConfig (state_data *myhandle,char *config)
 {
     FILE *tmp;
     int i = 0;
@@ -1140,5 +1550,139 @@ wvOpenConfig (char *config)
 	  config = HTMLCONFIG;
 	  tmp = fopen (config, "rb");
       }
-    return (tmp);
+    myhandle->path = config;
+    myhandle->fp = tmp;
+    return (tmp == NULL ? 0 : 1);
+}
+
+char * figure_name (wvParseStruct * ps)
+{
+  static int number;
+  static char * b_name = 0;
+  char * f_name = 0;
+  char buffer[10];
+
+  if (b_name == 0)
+    {
+      if (wv_arg_basename)
+        {
+          b_name = strdup (wv_arg_basename);
+#ifdef WV_REMOVE_SUFFIX
+          if (b_name) /* remove any suffix */
+            {
+              char * dot = 0;
+              char * ptr = b_name;
+              while (*ptr)
+                {
+                  if (*ptr == '.') dot = ptr;
+                  ptr++;
+                }
+              if (dot) *dot = 0;
+            }
+#endif /* WV_REMOVE_SUFFIX */
+        }
+      else
+        {
+          b_name = strdup (base_name (ps->filename));
+          if (b_name) /* remove '.doc' suffix; case insensitive */
+            {
+              if (strlen (b_name) >= 4)
+                {
+                  char * dot = b_name + strlen (b_name) - 4;
+                  if (strcasecmp (dot,".doc") == 0) *dot = 0;
+                }
+            }
+        }
+    }
+
+  if (b_name == 0)
+    {
+      fprintf (stderr,"error: unable to create basename!");
+      exit (1);
+    }
+
+  f_name = strdup (b_name);
+  if (f_name)
+    {
+      sprintf (buffer, "%d", number++);
+      wvAppendStr (&f_name, buffer);
+    }
+  else
+    {
+      fprintf (stderr,"error: unable to create filename!");
+      exit (1);
+    }
+
+  return (f_name);
+}
+
+char * name_to_url (char * name)
+{
+  static char * url = 0;
+  static long max = 0;
+  char * ptr = 0;
+  long count = 0;
+
+  ptr = name;
+  while (*ptr)
+    {
+      switch (*ptr)
+        {
+        case ' ':
+          count += 3;
+        break;
+        default:
+          count++;
+        break;
+        }
+      ptr++;
+    }
+  count++;
+
+  if (count > max)
+    {
+      char * more = 0;
+      if (url == 0)
+        {
+          more = malloc (count);
+        }
+      else
+        {
+          more = realloc (url,count);
+        }
+      if (more)
+        {
+          url = more;
+          max = count;
+        }
+    }
+
+  if (url)
+    {
+      count = 0;
+      ptr = name;
+      while (*ptr && (count < max))
+        {
+          switch (*ptr)
+            {
+            case ' ':
+              url[count++] = '%';
+              if (count < max) url[count++] = '2';
+              if (count < max) url[count++] = '0';
+            break;
+            default:
+              url[count++] = *ptr;
+            break;
+            }
+          ptr++;
+        }
+      url[max-1] = 0;
+    }
+  else
+    {
+      wvError (("failed to convert name to URL\n"));
+      return (name);
+    }
+
+  return (url);
 }
